SystemOrganization addCategory: #'SWE-IssueTracker'!

TestCase subclass: #ITTestGlorpDB
	instanceVariableNames: 'login accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestGlorpDB methodsFor: 'testing' stamp: 'afk 12/13/2007 14:52'!
setUp
	
	super setUp.
	login := ITGlorpDB login.
	accessor := DatabaseAccessor forLogin: login.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:34'!
testAddComment
	"Tests whether it is possible to add a comment into an existing issue."
	
	| original result session |
	session := (ITSession new) glorpSession.
	original := ITIssue new.
	original details: (ITIssueDetail with: 'title' with: 'tmp_testAddComment') asOrderedCollection.
	
	session inUnitOfWorkDo: [
		session register: original.
		original addComment: 'tmp_testAddComment_comment'.
	].
	
	result := session readOneOf: ITIssue where: [:each | each id = original id].
	
	self assert: (result comments first isKindOf: ITIssueComment) description: 'The Object is no ITIssueComment.'.
	self assert: (result comments first content) = 'tmp_testAddComment_comment' description: 'The comment is not the one just inserted.'.
	
	session delete: result.
! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:06'!
testAddRemoveIssue
	"Tests whether it is possible to add a new issue into the SQL database. Then try to remove it."
	
	| original result session |
	session := (ITSession new) glorpSession.
	
	original := ITIssue new.
	original details: (ITIssueDetail with: 'title' with: 'tmp_testAddRemoveIssue') asOrderedCollection.

	session inUnitOfWorkDo: [session register: original].
	
	result := session readOneOf: ITIssue where: [:each | each id = original id].
	
	self assert: ( result isKindOf: ITIssue ) description: 'Returning Object is not kind of ITIssue'.
	self deny: ( result id isNil ) description: 'Returning ID of issue is nil'.
	
	session delete: result.
	result := session readOneOf: ITIssue where: [:each | each id = original id].
	
	self assert: result isNil description: 'Issue was not deleted from database.'.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:47'!
testConnect
	"Tests whether ITGlorpDB>>#connect answers with a valid GlorpSession which could be used for operating with the SQL database."
	
	| session |
	session := ITGlorpDB connect.
	
	self assert: (login username = session currentLogin username) description: 'Username of session does not equal login username'.
	self assert: (login connectString = session currentLogin connectString) description: 'ConnectString of session does not equal login ConnectString'.
	self assert: (login password = session currentLogin password) description: 'Password of session does not equal login password '.
	
	self assert: session isLoggedIn.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:44'!
testLogin
	"Tests whether it is possible to use the default login for connecting to a vaild SQL database which is used in the whole application."
	
	self assert: accessor isLoggedIn not.
	accessor loginIfError: [:exception | self assert: false.].
	self assert: accessor isLoggedIn.
	accessor logout.
	self assert: accessor isLoggedIn not.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:45'!
testUniqueSession
	"Tests whether a glorp session really is a singleton."

	| session |
	session := ITSession new.
	self assert: (session glorpSession = nil) not description: 'Glorp session is nil.'.
	self assert: (session glorpSession == session glorpSession) description: 'Glorp session is not unique.'.! !

TestCase subclass: #ITTestImageDB
	instanceVariableNames: 'database'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestImageDB methodsFor: 'running' stamp: 'afk 12/13/2007 18:47'!
setUp

	super setUp.
	database := ITImageDB connect.! !

!ITTestImageDB methodsFor: 'running' stamp: 'mig 12/4/2007 18:30'!
tearDown
	super tearDown! !

!ITTestImageDB methodsFor: 'testing' stamp: 'mig 12/4/2007 18:39'!
testAddComment
	| issue comment | 
	
	issue := ITIssue new.
	issue := database addIssue: issue.
	database addComment: 'TESTCOMMENT' atID: issue id.
	issue := database getIssueWithID: issue id.
	
	comment := issue comments first.
	
	self assert: (comment isKindOf: ITIssueComment).
	self assert: (comment content) = 'TESTCOMMENT'.
	
	database deleteIssueWithID: issue id.! !

!ITTestImageDB methodsFor: 'testing' stamp: 'mig 12/4/2007 18:20'!
testAddRemoveIssue

	| newIssue returningIssue |
	newIssue := ITIssue new.

	"add"
	returningIssue := database addIssue: newIssue.
	self assert: ( returningIssue isKindOf: ITIssue ) description: 'Returning Object is not kind of ITIssue'.
	self deny: ( returningIssue id isNil ) description: 'Returning ID is nil'.
	
	"remove"
	database deleteIssueWithID: newIssue id.
	self assert: (database getIssueWithID: newIssue id) isNil description: 'Issue was not deleted from database.'.! !

!ITTestImageDB methodsFor: 'testing' stamp: 'afk 12/13/2007 18:53'!
testGetIssue
	
	"I'm testing wheter ImageDB is only returning a copy of the object or not."
	| id issue | 
	
	issue := ITIssue new.
	issue := database addIssue: issue.
	database addComment: 'TESTCOMMENT' atID: issue id.
	database setDetailWith: 'title' to: 'testGetIssue Title' atID: issue id.

	id := database getIssues first id.
	
	self deny: ( (database getIssueWithID: id) == (database getIssueWithID: id) ) description: 'ITImageDB is not giving only a copy of the Object'.
	
	database deleteIssueWithID: issue id.! !

!ITTestImageDB methodsFor: 'testing' stamp: 'mig 12/4/2007 18:08'!
testGetIssues

	| issues |

	issues := (database getIssues).
	
	self assert: ( (issues intersection: (database getIssues)) isEmpty ) description: 'Is not returning only a copy of the objects'.
	self assert: ( (issues reject: [:issue | issue isKindOf: ITIssue]) isEmpty ) description: 'Returns objects which are not of kind ITIssue'! !

!ITTestImageDB methodsFor: 'testing' stamp: 'afk 12/13/2007 18:14'!
testSetTag
	| issue | 
	
	issue := ITIssue new.
	issue := database addIssue: issue.
	database setDetailWith: 'TESTTAG' to: 'TESTVALUE' atID: issue id.
	issue := database getIssueWithID: issue id.
	
	self assert: (issue getTag: 'TESTTAG') = 'TESTVALUE'.
	
	database deleteIssueWithID: issue id.! !

TestCase subclass: #ITTestIssue
	instanceVariableNames: 'issue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:30'!
setUp

	issue := ITIssue new.! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:33'!
testAddComment
	
	issue addComment: 'TESTCOMMENT'.
	
	self 
		assert: issue comments last content = 'TESTCOMMENT' 
		description: 'A just added comment could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testComments
	"Tests whether the accessors for #comments do what they should do - set and get."
	
	| comments |
	comments := OrderedCollection new.
	issue comments: comments.
	
	self 
		assert: issue comments == comments 
		description: 'An ordered collection of comments was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testDetails
	"Tests whether the accessors for #details do what they should do - set and get."
	
	| details |
	details := OrderedCollection new.
	issue details: details.
	
	self 
		assert: issue details == details 
		description: 'An ordered collection of details was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testGetTag

	| detail |
	detail := ITIssueDetail new.
	detail tag: 'title'.
	detail content: 'TESTTITLE'.
	issue details: detail asOrderedCollection.
	
	self 
		assert: (issue getTag: 'title') = 'TESTTITLE' 
		description: 'A just added detail could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:30'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	issue id: 1.
	
	self
		assert: issue id = 1 
		description: 'An id was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:31'!
testSetDetailWithTo
	
	issue setDetailWith: 'title' to: 'TESTTITLE'.
	
	self 
		assert: (issue getTag: 'title') = 'TESTTITLE' 
		description: 'A new detail was set and could not be retrieved from the issue.'! !

TestCase subclass: #ITTestIssueComment
	instanceVariableNames: 'comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:28'!
setUp

	comment := ITIssueComment new.! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:29'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	comment content: 'TESTCOMMENT'.
	
	self 
		assert: comment content = 'TESTCOMMENT' 
		description: 'A content was just set and could not be retrieved from the comment.'! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:29'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	comment id: 1.
	
	self 
		assert: comment id = 1 
		description: 'An id was just set and could not be retrieved from the comment.'! !

TestCase subclass: #ITTestIssueDetail
	instanceVariableNames: 'detail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:25'!
setUp
	detail := ITIssueDetail new.! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:28'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	detail content: 'TESTCONTENT'.
	
	self 
		assert: detail content = 'TESTCONTENT' 
		description: 'A content was just set and could not be retrieved from the detail.'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:26'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	detail id: 1.
	
	self 
		assert: detail id = 1 
		description: 'An id was just set and could not be retrieved from the detail.'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:27'!
testTag
	"Tests whether the accessors for #tag do what they should do - set and get."

	detail tag: 'TESTTAG'.
	
	self 
		assert: detail tag = 'TESTTAG' 
		description: 'A tag was just set and could not be retrieved from the detail.'! !

TestCase subclass: #ITTestIssueManager
	instanceVariableNames: 'issueManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueManager methodsFor: 'testing' stamp: 'afk 12/15/2007 12:10'!
setUp
	. 
	issueManager := ITIssueManager new.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:47'!
testAddCommentAtID

	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testAddCommentAtID'.
	
	issueManager addComment: 'tmp_testAddCommentAtID_comment' atID: issue id.
	
	self 
		assert: (issueManager getIssueWithID: issue id) comments last content = 'tmp_testAddCommentAtID_comment'
		description: 'A comment to a new issue was just set and could not be retrieved through an issue manager.'.
	
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:55'!
testAddIssueWithTitle
	
	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testAddIssueWithTitle'.
	
	self 
		assert: (issue isKindOf: ITIssue) 
		description: 'ITIssueManager>>#addIssueWithTitle: is not returning an object with type ITIssue'.
		
	self
		deny: (issue id) isNil
		description: 'ITIssueManager>>#addIssueWithTitle: returns an ITIssue object which has no ID.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testAddIssueWithTitle'
		description: 'A new issue with a title was just set and the title could not be retrieved through an issue manager.'.
	
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 11:00'!
testAddIssueWithTitleWithComment

	| issue |
	issue := issueManager
		addIssueWithTitle: 'tmp_testAddIssueWithTitleWithComment'
		withComment: 'tmp_testAddIssueWithTitleWithComment_comment'.
	
	self 
		assert: (issue isKindOf: ITIssue) 
		description: 'ITIssueManager>>#addIssueWithTitle:WithComment: is not returning an object with type ITIssue'.
		
	self
		deny: (issue id) isNil
		description: 'ITIssueManager>>#addIssueWithTitle:WithComment: returns an ITIssue object which has no ID.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testAddIssueWithTitleWithComment'
		description: 'A new issue with a title and a comment was just set and the title could not be retrieved through an issue manager.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) comments last) = 'tmp_testAddIssueWithTitleWithComment_comment'
		description: 'A new issue with a title and a comment was just set and the comment could not be retrieved through an issue manager.'.		
	
	issueManager deleteIssueWithID: issue id.
! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:51'!
testDeleteIssueWithID

	| issue |
	
	issue := issueManager addIssueWithTitle: 'tmp_testDeleteIssueWithID'.
	
	issueManager deleteIssueWithID: issue id.
	
	self
		assert: (issueManager getIssueWithID: issue id) isNil
		description: 'A new issue was just added and could not be deleted through an issue manager.'
	

	! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:54'!
testGetIssueWithID
	
	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testGetIssueWithID'.
	
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testGetIssueWithID'
		description: 'A new issue was just added and could not be retrieved through an issue manager'.
		
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 11:04'!
testGetIssues
	
	issueManager getIssues do: [ :issue |
		self 
			assert: (issue isKindOf: ITIssue)
			description: 'ITIssueManager>>#getIssues returns a collection in where not all objects are issues.'.
		
		self
			deny: (issue id) isNil
			description: 'ITIssueManager>>#getIssues returns a collection in where not all issues have an ID.'.
	].
	
	
	! !

WAComponent subclass: #ITComponentAddComment
	instanceVariableNames: 'comment issueID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddComment class methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 19:36'!
newWith: anID
	| newInstance |
	newInstance := self new.
	newInstance issueID: anID.
	^newInstance.! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment
	^ comment! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment: anObject
	comment := anObject! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:36'!
issueID
	^ issueID! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:36'!
issueID: anObject
	issueID := anObject! !

!ITComponentAddComment methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:35'!
renderContentOn: html
	html form: [
		html textArea on: #comment of: self.
		html break.
		html submitButton
			callback: [
				| issue issueManager |
				issueManager := self session issueManager.
				issue := issueManager getIssueWithID: self issueID.
				
				issueManager addComment: self comment atID: self issueID.
				
				"old ImageDB stuff TODO: Delete" 
				"issue addComment: self comment.
				issueManager saveChanges: issue."
			];
			text: 'Add Comment'.
	]! !

WAComponent subclass: #ITComponentAddIssue
	instanceVariableNames: 'title initialComment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment
	^ initialComment! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment: anObject
	initialComment := anObject! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mig 12/10/2007 17:48'!
renderContentOn: html
	html heading 
		level: 1;
		with: 'Add Issue'.
	html form: 
		[
		html text: 'Title'.
			html break.
		html textInput on: #title of: self.
			html break.
		html text: 'Description'.
			html break.
		html textArea on: #initialComment of: self.
			html break.
		html submitButton 
			callback: [
				self session issueManager addIssueWithTitle: self title withComment: self initialComment.
				self answer. 
			];
			value: 'add Issue'.
		html cancelButton
			callback: [self answer];
			value: 'cancel'.
		]! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title
	^ title! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title: anObject
	title := anObject! !

WAComponent subclass: #ITComponentDetails
	instanceVariableNames: 'issueID addcomment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentDetails class methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 19:36'!
newWith: anID
	|view|
	view := self new.
	view issueID: anID.
	^view.! !

!ITComponentDetails methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 19:16'!
children
	^ Array with: addcomment.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:34'!
issueID
	^ issueID! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:34'!
issueID: anObject
	issueID := anObject! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mig 12/10/2007 17:49'!
renderContentOn: html

	| issue |
	
	issue := self session issueManager getIssueWithID: self issueID.
	addcomment := ITComponentAddComment newWith: self issueID.

	html heading
		level: 2;
		with: 'detailed view of an issue'.
	html div
		with: [
			html text: issue id.
			html break.
			html text: (issue getTag: 'title').
			
			html break; break.
			html table: [
				(issue comments) do: [:comment|
					html tableRow: [ html tableData: [ html text: comment content ] ].

				].
				html tableRow: [ 
					html tableData with: addcomment.
				].
			].
			html anchor
				callback: [self answer];
				with: [html text: 'return to list'].
		]
					
				! !

WAComponent subclass: #ITComponentListIssues
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentListIssues methodsFor: 'rendering' stamp: 'afk 12/13/2007 21:54'!
renderContentOn: html
	| issues |
	issues := self session issueManager getIssues.
	html heading 
		level: 2;
		with: 'List all issues'.
	html form:[
		html table:[
			html tableRow: [
				html tableData: 'id'.
				html tableData: 'title'.
				html tableData: 'details'.
				html tableData: 'delete'.
				].
			issues do: [:issue|
				html tableRow: [
					html tableData: (issue id asString).
					html tableData: (issue getTag: 'title').
					html tableData: [
						html anchor 
							callback: [self call: (ITComponentDetails newWith: issue id)];
							with: [html text: 'detailed view'].
						].
					html tableData: [
						html anchor
							callback: [((self session issueManager) deleteIssueWithID: issue id)];
							with: [html text: 'delete'].
						].
					].
				].
			].
		].! !

WAComponent subclass: #ITComponentMenu
	instanceVariableNames: 'listIssues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentMenu class methodsFor: 'as yet unclassified' stamp: 'mt 11/23/2007 18:52'!
canBeRoot
	^true.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mt 11/23/2007 19:05'!
children
	^Array with: self listIssues.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mt 11/23/2007 19:05'!
initialize
	super initialize.
	self listIssues: ITComponentListIssues new.! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:05'!
listIssues
	^ listIssues! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:05'!
listIssues: anObject
	listIssues := anObject! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'mt 11/23/2007 19:51'!
renderContentOn: html
	html heading
		level: 1;
		with: 'IssueTracker'.
	html anchor 
		callback: [ self call: ITComponentAddIssue new ]; 
		with: [ html text: 'add issue'].
	html div
		with: self listIssues.! !

Object subclass: #ITGlorpDB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDB commentStamp: 'mt 12/14/2007 10:22' prior: 0!
I encapsulate the connection to a SQL database, at the moment with PostGreSQL. Feel free to use my class side method #connect to get a new ITGlorpSession instance. But be aware of the connection. You'll have to check whether ITGlorpSession>>#isLoggedIn.!

!ITGlorpDB class methodsFor: 'debug' stamp: 'mt 12/14/2007 10:37'!
closeAllConnections
	ITGlorpSession allInstances do: [:aSession |
		aSession logout.
		Transcript show: 'DEBUG: You closed a Session.']! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 12/15/2007 12:30'!
connect
	"Connects to the database with the given information from login and returns the current (new) session."	
	
	| accessor session system |
	accessor := DatabaseAccessor forLogin: self login.
	accessor login.

	session := GlorpSession new.
	session accessor: accessor.

	system := ITGlorpDescriptorSystem new.
	session system: system.
	system platform: session platform.
	
	^ session
! !

!ITGlorpDB class methodsFor: 'debug' stamp: 'mt 12/14/2007 10:37'!
createTables
	"I'm creating alle relevant tables but only if there are none of them."
	
	| session |
	Transcript show: 'DEBUG: You try to create all tables again.'.
	session := ITGlorpDB connect.
	session inTransactionDo: [session system allTables do:
		[:each | session accessor createTable: each ifError: [:error | Transcript show: error messageText]]].! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 12/14/2007 10:23'!
login
	"The default login can be found here. It is used across the whole issue tracker application. It is also used for unit tests."
	
	^ Login new database: PostgreSQLPlatform new;
		username: 'issuetracker';
		password: 'weloveseasidedontwe';
		connectString: 'theresnoplacelike.homeftp.org:5432_issuetracker'.! !

Object subclass: #ITImageDB
	instanceVariableNames: ''
	classVariableNames: 'ISSUES NEXTID'
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITImageDB class methodsFor: 'as yet unclassified' stamp: 'afk 11/12/2007 16:11'!
connect

	^ self new.! !

!ITImageDB methodsFor: 'Issues' stamp: 'mig 11/30/2007 18:29'!
addComment: aComment atID: id
	| issue |
	issue := 	(ISSUES at: id).
	issue comments add: (ITIssueComment newWithID: issue nextcommentid WithString: aComment).
	issue nextcommentid: issue nextcommentid+1.! !

!ITImageDB methodsFor: 'Issues' stamp: 'afk 11/30/2007 14:17'!
addIssue: anIssue

	anIssue id: NEXTID.
	ISSUES at: anIssue id put: anIssue.
	NEXTID := NEXTID+1.
	
	^ self getIssueWithID: anIssue id.! !

!ITImageDB methodsFor: 'initialize-release' stamp: 'mig 11/30/2007 18:50'!
clear

	ISSUES := Dictionary new.
	NEXTID := 0.
	! !

!ITImageDB methodsFor: 'Issues' stamp: 'afk 11/30/2007 14:30'!
deleteIssueWithID: id
	ISSUES removeKey: id.! !

!ITImageDB methodsFor: 'Issues' stamp: 'mig 12/4/2007 18:17'!
getIssueWithID: id

	"I'm returning a copy of the ITIssue gequested by the given id"
	^ (ISSUES at: id ifAbsent: nil) veryDeepCopy.
	! !

!ITImageDB methodsFor: 'Issues' stamp: 'mig 11/30/2007 18:30'!
getIssues
	^ ISSUES collect: [:anIssue | self getIssueWithID: anIssue id]
	! !

!ITImageDB methodsFor: 'initialize-release' stamp: 'afk 12/13/2007 18:25'!
initialize

	ISSUES ifNil: [ISSUES := Dictionary new].
	NEXTID ifNil: [NEXTID := 0].
	super initialize.! !

!ITImageDB methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 18:11'!
setDetailWith: aTag to: aValue atID: anID
	(ISSUES at: anID) details add: (ITIssueDetail with: aTag with: aValue).! !

Object subclass: #ITIssue
	instanceVariableNames: 'id details comments nextcommentid changeset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssue class methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:12'!
with: aTitle with: aComment

	| newIssue comment |
	newIssue := self new.
	comment := ITIssueComment new.
	
	comment content: aComment.
	
	newIssue comments add: comment.
	newIssue details add: (ITIssueDetail with: 'title' with: aTitle).

	^ newIssue! !

!ITIssue methodsFor: 'changing' stamp: 'mt 12/14/2007 16:01'!
addComment: aString

	self comments add: (ITIssueComment with: aString).
	"self changeset add: (Array with: #addComment:atID: with: aComment with: self id)."! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:34'!
changeset
	^ changeset! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:34'!
changeset: anObject
	changeset := anObject! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:55'!
comments
	^comments.! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 12/11/2007 15:23'!
comments: aCollection
	comments := aCollection.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details
	^ details! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details: anObject
	details := anObject! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 18:18'!
getTag: tag
	details do: 
		[:each | 
		each tag = tag
			ifTrue: [^ each content]
		].
	^nil.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 16:20'!
id
	^ id! !

!ITIssue methodsFor: 'changing' stamp: 'afk 11/12/2007 16:20'!
id: anObject
	id := anObject! !

!ITIssue methodsFor: 'initialize-release' stamp: 'afk 12/13/2007 18:15'!
initialize
	changeset := OrderedCollection new.
	tags := Dictionary new.
	details := OrderedCollection new.
	comments := OrderedCollection new.
	changeset := OrderedCollection new.
	nextcommentid := 0.
	super initialize! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:29'!
nextcommentid
	^ nextcommentid! !

!ITIssue methodsFor: 'accessing' stamp: 'mig 11/30/2007 18:29'!
nextcommentid: anObject
	nextcommentid := anObject! !

!ITIssue methodsFor: 'changing' stamp: 'afk 12/13/2007 18:01'!
setDetailWith: aTag to: aValue
	self changeset add: (Array with: #setDetailWith:to:atID: with: aTag with: aValue with: self id)! !

Object subclass: #ITIssueComment
	instanceVariableNames: 'id content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueComment class methodsFor: 'as yet unclassified' stamp: 'mig 11/30/2007 18:22'!
newWithID: aNumber WithString: aString
	^ self new
		id: aNumber;
		content: aString;
		yourself.! !

!ITIssueComment class methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 20:52'!
with: aComment
	^ self new
		content: aComment;
		yourself.! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content
	^ content! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content: anObject
	content := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id
	^ id! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id: anObject
	id := anObject! !

!ITIssueComment methodsFor: 'initialize-release' stamp: 'mt 12/14/2007 10:02'!
initialize
	
	super initialize.
	
	content := ''.! !

Object subclass: #ITIssueDetail
	instanceVariableNames: 'id tag content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueDetail class methodsFor: 'as yet unclassified' stamp: 'afk 12/13/2007 18:10'!
with: aTag with: aValue
	
	| detail |
	detail := ITIssueDetail new.
	detail tag: aTag.
	detail content: aValue.
	
	^ detail.! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content
	^ content! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content: anObject
	content := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id
	^ id! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id: anObject
	id := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag
	^ tag! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag: anObject
	tag := anObject! !

Object subclass: #ITIssueManager
	instanceVariableNames: 'database'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 12/15/2007 12:34'!
addComment: aString atID: aNumber
	"Adds a new ITIssueComment object with the content 'aString' to the issue with the ID 'aNumber'."

	| issue glorpSession |
	glorpSession := ITGlorpSession getInstance.
	issue := glorpSession readOneOf: ITIssue where: [:each | each id = aNumber].
	
	glorpSession inUnitOfWorkDo: [
		glorpSession register: issue.
		issue addComment: aString.
	].! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 12/15/2007 12:35'!
addIssueWithTitle: aString
	"Adds a new issue with a detail 'title' to the database. Returns this new issue."
	
	| session issue |
	session := ITGlorpSession getInstance.
	
	issue := ITIssue new.
	issue setDetailWith: 'title' to: aString.
	
	session inUnitOfWorkDo: [
		session register: issue.
	].

	^issue.! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 12/15/2007 12:35'!
addIssueWithTitle: aTitleString withComment: aCommentString
	"Adds a new issue with a title and an initial comment to the database. Returns this new issue."
	
	| session issue |
	session := ITGlorpSession getInstance.
	
	issue := ITIssue with: aTitleString with: aCommentString.
	
	session inUnitOfWorkDo: [
		session register: issue.
	].
	
	^issue.! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 12/15/2007 12:35'!
deleteIssueWithID: aNumber
	"Tries to delete an issue with the given ID from the database."
	
	| session issue |
	session := ITGlorpSession getInstance.
	
	issue := session readOneOf: ITIssue where: [:each | each id = aNumber].
	
	session delete: issue.
	! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2007 12:35'!
getIssueWithID: aNumber
	"Returns an ITIssue object where the 'id' equals 'aNumber'."

	| glorpSession |
	glorpSession := ITGlorpSession getInstance.
	^ glorpSession readOneOf: ITIssue where: [:each | each id = aNumber].
! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 12/15/2007 12:35'!
getIssues
	"Returns all ITIssue objects in a collection that are in the SQL database"

	| glorpSession |
	glorpSession := ITGlorpSession getInstance.

	^ glorpSession readManyOf: ITIssue.! !

!ITIssueManager methodsFor: 'initialize-release' stamp: 'afk 11/12/2007 16:22'!
initialize
	
	database := ITImageDB connect.
	super initialize.! !

!ITIssueManager methodsFor: 'changing' stamp: 'mig 11/30/2007 18:44'!
saveChanges: anIssue

	anIssue changeset do: [ :change |
		database perform: (change first) withArguments: (change allButFirst).
	]
! !

Object subclass: #ITRole
	instanceVariableNames: 'name rights'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:34'!
hasRight: aSymbol
	^ rights contains: aSymbol.! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
name1
	^ name! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
name: anObject
	name := anObject! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
rights
	^ rights! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:32'!
rights: anObject
	rights := anObject! !

Object subclass: #ITSecurityManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

Object subclass: #ITUser
	instanceVariableNames: 'username passwordHash roles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash
	^ passwordHash! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash: anObject
	passwordHash := anObject! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles
	^ roles! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles: anObject
	roles := anObject! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username
	^ username! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username: anObject
	username := anObject! !

WASession subclass: #ITSession
	instanceVariableNames: 'issueManager securityManager glorpSession user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSession methodsFor: 'accessing' stamp: 'mt 12/15/2007 12:31'!
glorpSession
	"Returns the glorp session for this session"
	
	^ ITGlorpSession getInstance.
			! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:45'!
issueManager
	issueManager ifNil: [
		issueManager := ITIssueManager new.
	].
	
	^issueManager.! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
securityManager
	securityManager ifNil: [
		securityManager := ITSecurityManager new.
	].
	
	^securityManager.! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user
	^ user! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user: anObject
	user := anObject! !

GlorpSession subclass: #ITGlorpSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession commentStamp: 'mt 12/15/2007 10:12' prior: 0!
I'm a usual GlorpSession and control a cache manager, a description system and all unit of works. Every manipulation of the SQL database goes through my "fingers".

Use #inUnitOfWorkDo: for watching over "dirty" objects and change all relevant data in the database. But DO NOT forget to register them with #register inside a unit of work.

Any read of data doesn't have to be embedded in a unit of work. Just use the result of #read:, #readOneOf:, #readManyOf:, #read:where: a. s. o.

If you want to delete an object you doesn't have to use a unit of work either.!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2007 12:31'!
getInstance
	"Using singleton pattern to ensure that there is only one instance of ITGlorpSession"

	instance ifNil:
		[
		instance := ITGlorpDB connect.
		].
	^ instance! !

!ITGlorpSession class methodsFor: 'as yet unclassified' stamp: 'afk 12/15/2007 12:00'!
new

	^ Exception new messageText: 'Use getInstance because ITGlorpSession is a singleton'.! !

DescriptorSystem subclass: #ITGlorpDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDescriptorSystem commentStamp: 'mt 12/14/2007 10:23' prior: 0!
I'm the descriptor system that should be used in every new ITGlorpSession. I'm responsible for all mappings between the IT-Classes and the relevant tables on the SQL server.

You should modify #allTableNames and #constructAllClasses whenever you want to add new objects of the domain space for usage here.

#tableNamed<TABLE_NAME>    ... descriptions about the table structure on the server
#descriptorFor<CLASS_NAME> ... defines mappings between classes and tables
#classModelFor<CLASS_NAME> ... defines types (classes) for symbols (attributes) used by mappings

The relational schemes can be found in the comments for any #tableNamed<TABLE_NAME>. Primary keys are underlined, e.g. SCHEME(ATTRIBUTE_A, ATTRIBUTE_B)
                        -----------!

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'mt 12/14/2007 09:34'!
allTableNames
	"Here are all tables listed which hold all persistent data."
	
	^ #('ISSUE' 'COMMENT' 'DETAIL')! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:54'!
classModelForITIssue: aClassModel
	"The unique ID can be up to 1.073.741.823 - should be enough.
	 -> SmallInteger maxVal
	 4-Byte-Integers (signed) are used in the database"
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #comments collection: OrderedCollection  of: ITIssueComment.
	aClassModel newAttributeNamed: #details collection: OrderedCollection  of: ITIssueDetail.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueComment: aClassModel
	"See #classModelForITIssue for explanation of ID."
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueDetail: aClassModel
	"See #classModelForITIssue for explanation of ID."

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #tag type: String.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'mt 12/14/2007 09:35'!
constructAllClasses
	"Here are all classes listed which are necessary for the mappings."
	
	^ (super constructAllClasses)
		add: ITIssue;
		add: ITIssueComment;
		add: ITIssueDetail;
		yourself! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:45'!
descriptorForITIssue: aDescriptor
	"Every issue contains a unique ID, a collection of comments and a collection of details."
	
	| issueTable commentTable detailTable |
	issueTable   := self tableNamed: 'ISSUE'.
	commentTable := self tableNamed: 'COMMENT'.
	detailTable  := self tableNamed: 'DETAIL'.

	aDescriptor table: issueTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (issueTable fieldNamed: 'ID').

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #comments;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to: (commentTable	fieldNamed: 'ISSUE_ID')
		).
	
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #details;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to:  (detailTable fieldNamed: 'ISSUE_ID')
		).! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueComment: aDescriptor
	"Every comment contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'COMMENT'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueDetail: aDescriptor
	"Every detail contains a unique ID, a tag and its content."

	| table |
	table := self tableNamed: 'DETAIL'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #tag to: (table fieldNamed: 'TAG').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:59'!
tableForCOMMENT: aTable
	"COMMENTS(ID, ISSUE_ID, TIMESTAMP, CONTENT)
	          --"
	
	| issueId |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueId := aTable createFieldNamed: 'ISSUE_ID' type: (platform int4).
	aTable createFieldNamed: 'TIMESTAMP' type: (platform timestamp).
	aTable createFieldNamed: 'CONTENT' type: (platform text).
	
	"foreign key definition for the IDs of issues"
	aTable addForeignKeyFrom: issueId 
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID').
		! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForDETAIL: aTable
	"DETAIL(ID, ISSUE_ID, TAG, CONTENT)
	       --"
	
	| issueID |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueID := (aTable createFieldNamed: 'ISSUE_ID' type: (platform int4)).
	aTable createFieldNamed: 'TAG' type: (platform varchar: 255).
	aTable createFieldNamed: 'CONTENT' type: (platform varchar: 255).
	
	"foreign key definition for IDs of issues"
	aTable addForeignKeyFrom: issueID to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID').! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForISSUE: aTable
	"ISSUE(ID)
	       --"
	
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.! !
